Alsa-Devel Archive on lore.kernel.org
 help / color / mirror / Atom feed
From: Mark Brown <broonie@opensource.wolfsonmicro.com>
To: Takashi Iwai <tiwai@suse.de>
Cc: alsa-devel@alsa-project.org,
	Mark Brown <broonie@opensource.wolfsonmicro.com>
Subject: [PATCH 11/17] ASoC: cs4270: add Master Playback Switch
Date: Tue, 28 Apr 2009 16:23:36 +0100	[thread overview]
Message-ID: <1240932222-3718-11-git-send-email-broonie@opensource.wolfsonmicro.com> (raw)
In-Reply-To: <20090428152256.GB2244@rakim.wolfsonmicro.main>

From: Daniel Mack <daniel@caiaq.de>

This adds a new control named 'Master Playback Switch' for cs4270
codecs. It is implemented using the new SOC_DOUBLE_EXT macro to catch
the put function and store the information about manually set mute
controls from userspace. When a manual mute is set, we don't want the
soc core to un-mute the outputs.

Renamed cs4270_mute() to cs4270_dai_mute() to avoid confusion.

Signed-off-by: Daniel Mack <daniel@caiaq.de>
Acked-by: Timur Tabi <timur@freescale.com>
Signed-off-by: Mark Brown <broonie@opensource.wolfsonmicro.com>
---
 sound/soc/codecs/cs4270.c |   44 +++++++++++++++++++++++++++++++++++++++-----
 1 files changed, 39 insertions(+), 5 deletions(-)

diff --git a/sound/soc/codecs/cs4270.c b/sound/soc/codecs/cs4270.c
index 3c34fe6..ece6ed6 100644
--- a/sound/soc/codecs/cs4270.c
+++ b/sound/soc/codecs/cs4270.c
@@ -109,6 +109,7 @@ struct cs4270_private {
 	unsigned int mclk; /* Input frequency of the MCLK pin */
 	unsigned int mode; /* The mode (I2S or left-justified) */
 	unsigned int slave_mode;
+	unsigned int manual_mute;
 };
 
 /**
@@ -453,7 +454,7 @@ static int cs4270_hw_params(struct snd_pcm_substream *substream,
 }
 
 /**
  * cs4270_mute - enable/disable the CS4270 external mute
  * @dai: the SOC DAI
  * @mute: 0 = disable mute, 1 = enable mute
  *
@@ -462,21 +463,52 @@ static int cs4270_hw_params(struct snd_pcm_substream *substream,
  * board does not have the MUTEA or MUTEB pins connected to such circuitry,
  * then this function will do nothing.
  */
-static int cs4270_mute(struct snd_soc_dai *dai, int mute)
+static int cs4270_dai_mute(struct snd_soc_dai *dai, int mute)
 {
 	struct snd_soc_codec *codec = dai->codec;
+	struct cs4270_private *cs4270 = codec->private_data;
 	int reg6;
 
 	reg6 = snd_soc_read(codec, CS4270_MUTE);
 
 	if (mute)
 		reg6 |= CS4270_MUTE_DAC_A | CS4270_MUTE_DAC_B;
-	else
+	else {
 		reg6 &= ~(CS4270_MUTE_DAC_A | CS4270_MUTE_DAC_B);
+		reg6 |= cs4270->manual_mute;
+	}
 
 	return snd_soc_write(codec, CS4270_MUTE, reg6);
 }
 
+/**
+ * cs4270_soc_put_mute - put callback for the 'Master Playback switch'
+ * 			 alsa control.
+ * @kcontrol: mixer control
+ * @ucontrol: control element information
+ *
+ * This function basically passes the arguments on to the generic
+ * snd_soc_put_volsw() function and saves the mute information in
+ * our private data structure. This is because we want to prevent
+ * cs4270_dai_mute() neglecting the user's decision to manually
+ * mute the codec's output.
+ *
+ * Returns 0 for success.
+ */
+static int cs4270_soc_put_mute(struct snd_kcontrol *kcontrol,
+				struct snd_ctl_elem_value *ucontrol)
+{
+	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);
+	struct cs4270_private *cs4270 = codec->private_data;
+	int left = !ucontrol->value.integer.value[0];
+	int right = !ucontrol->value.integer.value[1];
+
+	cs4270->manual_mute = (left ? CS4270_MUTE_DAC_A : 0) |
+			      (right ? CS4270_MUTE_DAC_B : 0);
+
+	return snd_soc_put_volsw(kcontrol, ucontrol);
+}
+
 /* A list of non-DAPM controls that the CS4270 supports */
 static const struct snd_kcontrol_new cs4270_snd_controls[] = {
 	SOC_DOUBLE_R("Master Playback Volume",
@@ -486,7 +518,9 @@ static const struct snd_kcontrol_new cs4270_snd_controls[] = {
 	SOC_SINGLE("Zero Cross Switch", CS4270_TRANS, 5, 1, 0),
 	SOC_SINGLE("Popguard Switch", CS4270_MODE, 0, 1, 1),
 	SOC_SINGLE("Auto-Mute Switch", CS4270_MUTE, 5, 1, 0),
-	SOC_DOUBLE("Master Capture Switch", CS4270_MUTE, 3, 4, 1, 1)
+	SOC_DOUBLE("Master Capture Switch", CS4270_MUTE, 3, 4, 1, 1),
+	SOC_DOUBLE_EXT("Master Playback Switch", CS4270_MUTE, 0, 1, 1, 1,
+		snd_soc_get_volsw, cs4270_soc_put_mute),
 };
 
 /*
@@ -506,7 +540,7 @@ static struct snd_soc_dai_ops cs4270_dai_ops = {
 	.hw_params	= cs4270_hw_params,
 	.set_sysclk	= cs4270_set_dai_sysclk,
 	.set_fmt	= cs4270_set_dai_fmt,
-	.digital_mute	= cs4270_mute,
+	.digital_mute	= cs4270_dai_mute,
 };
 
 struct snd_soc_dai cs4270_dai = {
@@ -453,7 +454,7 @@ static int cs4270_hw_params(struct snd_pcm_substream *substream,
 
 /**
- * cs4270_mute - enable/disable the CS4270 external mute
+ * cs4270_dai_mute - enable/disable the CS4270 external mute
  * @dai: the SOC DAI
  * @mute: 0 = disable mute, 1 = enable mute
-- 
1.6.2.4

  parent reply	other threads:[~2009-04-28 15:23 UTC|newest]

Thread overview: 36+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2009-04-28 15:22 ASoC updates for 2.6.31 Mark Brown
2009-04-28 15:23 ` [PATCH 01/17] ASoC: Beagle: Add support for 4 channel Mark Brown
2009-04-28 15:23 ` [PATCH 02/17] ASoC: add SOC_DOUBLE_EXT macro Mark Brown
2009-04-28 15:23 ` [PATCH 03/17] ASoC: Fix S3C64xx IIS device registration and support both ports Mark Brown
2009-04-28 15:23 ` [PATCH 04/17] ASoC: S3C2412: Failing to get the I2S clock is an error Mark Brown
2009-04-28 15:23 ` [PATCH 05/17] ASoC: Enforce symmetric rates for S3C64xx I2S interface Mark Brown
2009-04-28 15:23 ` [PATCH 06/17] ASoC: Include WM8350 register definitions in CODEC header Mark Brown
2009-04-28 15:23 ` [PATCH 07/17] ASoC: s3c-i2s-v2 diagnostic improvements Mark Brown
2009-04-28 15:23 ` [PATCH 08/17] ASoC: Use our registration function for S3C64xx Mark Brown
2009-04-28 15:23 ` [PATCH 09/17] ASoC WM8940 Driver Mark Brown
2009-04-28 15:23 ` [PATCH 10/17] ASoC: cs4270: fix Master Capture Switch polarity Mark Brown
2009-04-28 15:23 ` Mark Brown [this message]
2009-04-28 15:23 ` [PATCH 12/17] ASoC: Fix logic in WM8350 master clocking check Mark Brown
2009-04-28 15:23 ` [PATCH 13/17] ASoC: Set the MPC5200 i2s driver to BROKEN status Mark Brown
2009-04-28 15:23 ` [PATCH 14/17] ASoC: Staticise TLV values in WM8940 Mark Brown
2009-04-28 15:23 ` [PATCH 15/17] ASoC: TWL4030: Fix gain control for earpiece amplifier Mark Brown
2009-04-28 15:23 ` [PATCH 16/17] ASoC: Basic split of mpc5200 DMA code out from mpc5200_psc_i2s Mark Brown
2009-04-28 15:46   ` Takashi Iwai
2009-04-28 15:59     ` Jon Smirl
2009-04-28 16:51       ` Takashi Iwai
2009-04-28 16:55         ` Jon Smirl
2009-04-28 17:14           ` Jon Smirl
2009-04-28 19:37             ` Takashi Iwai
2009-04-28 19:53               ` Mark Brown
2009-04-28 19:59                 ` Jon Smirl
2009-04-28 20:16                   ` Jon Smirl
2009-04-28 21:01                   ` Mark Brown
2009-04-28 22:06                   ` Takashi Iwai
2009-04-28 21:59                 ` Takashi Iwai
2009-04-28 19:35           ` Takashi Iwai
2009-04-28 19:37           ` Mark Brown
2009-04-28 19:25     ` Mark Brown
2009-04-28 19:32       ` Jon Smirl
2009-04-28 19:50         ` Mark Brown
2009-04-28 21:52       ` Takashi Iwai
2009-04-28 15:23 ` [PATCH 17/17] ASoC: Rename the PSC functions to DMA Mark Brown

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1240932222-3718-11-git-send-email-broonie@opensource.wolfsonmicro.com \
    --to=broonie@opensource.wolfsonmicro.com \
    --cc=alsa-devel@alsa-project.org \
    --cc=tiwai@suse.de \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox